@@ -1,5 +1,5 @@
 /* server.c --- SASL mechanism GSSAPI as defined in RFC 4752, server side.
- * Copyright (C) 2002-2012 Simon Josefsson
+ * Copyright (C) 2002-2020 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -21,7 +21,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#include "config.h"
 #endif
 
 /* Get malloc, free. */
@@ -34,11 +34,11 @@
 #include "x-gssapi.h"
 
 #ifdef HAVE_LIBGSS
-# include <gss.h>
+#include <gss.h>
 #elif HAVE_GSSAPI_H
-# include <gssapi.h>
+#include <gssapi.h>
 #elif HAVE_GSSAPI_GSSAPI_H
-# include <gssapi/gssapi.h>
+#include <gssapi/gssapi.h>
 #endif
 
 #include "gss-extra.h"
@@ -56,57 +56,13 @@ int
 _gsasl_gssapi_server_start (Gsasl_session * sctx, void **mech_data)
 {
   _Gsasl_gssapi_server_state *state;
-  OM_uint32 maj_stat, min_stat;
-  gss_name_t server;
-  gss_buffer_desc bufdesc;
-  const char *service;
-  const char *hostname;
-
-  service = gsasl_property_get (sctx, GSASL_SERVICE);
-  if (!service)
-    return GSASL_NO_SERVICE;
-
-  hostname = gsasl_property_get (sctx, GSASL_HOSTNAME);
-  if (!hostname)
-    return GSASL_NO_HOSTNAME;
-
-  /* FIXME: Use asprintf. */
-
-  bufdesc.length = strlen (service) + strlen ("@") + strlen (hostname) + 1;
-  bufdesc.value = malloc (bufdesc.length);
-  if (bufdesc.value == NULL)
-    return GSASL_MALLOC_ERROR;
-
-  sprintf (bufdesc.value, "%s@%s", service, hostname);
 
   state = (_Gsasl_gssapi_server_state *) malloc (sizeof (*state));
   if (state == NULL)
-    {
-      free (bufdesc.value);
-      return GSASL_MALLOC_ERROR;
-    }
-
-  maj_stat = gss_import_name (&min_stat, &bufdesc, GSS_C_NT_HOSTBASED_SERVICE,
-			      &server);
-  free (bufdesc.value);
-  if (GSS_ERROR (maj_stat))
-    {
-      free (state);
-      return GSASL_GSSAPI_IMPORT_NAME_ERROR;
-    }
-
-  maj_stat = gss_acquire_cred (&min_stat, server, 0,
-			       GSS_C_NULL_OID_SET, GSS_C_ACCEPT,
-			       &state->cred, NULL, NULL);
-  gss_release_name (&min_stat, &server);
-
-  if (GSS_ERROR (maj_stat))
-    {
-      free (state);
-      return GSASL_GSSAPI_ACQUIRE_CRED_ERROR;
-    }
+    return GSASL_MALLOC_ERROR;
 
   state->step = 0;
+  state->cred = GSS_C_NO_CREDENTIAL;
   state->context = GSS_C_NO_CONTEXT;
   state->client = NULL;
   *mech_data = state;
@@ -118,7 +74,7 @@ int
 _gsasl_gssapi_server_step (Gsasl_session * sctx,
 			   void *mech_data,
 			   const char *input, size_t input_len,
-			   char **output, size_t * output_len)
+			   char **output, size_t *output_len)
 {
   _Gsasl_gssapi_server_state *state = mech_data;
   gss_buffer_desc bufdesc1, bufdesc2;
@@ -134,11 +90,48 @@ _gsasl_gssapi_server_step (Gsasl_session * sctx,
   switch (state->step)
     {
     case 0:
-      if (input_len == 0)
-	{
-	  res = GSASL_NEEDS_MORE;
-	  break;
-	}
+      {
+	gss_name_t server;
+	const char *service;
+	const char *hostname;
+
+	if (input_len == 0)
+	  {
+	    res = GSASL_NEEDS_MORE;
+	    break;
+	  }
+
+	service = gsasl_property_get (sctx, GSASL_SERVICE);
+	if (!service)
+	  return GSASL_NO_SERVICE;
+
+	hostname = gsasl_property_get (sctx, GSASL_HOSTNAME);
+	if (!hostname)
+	  return GSASL_NO_HOSTNAME;
+
+	/* FIXME: Use asprintf. */
+
+	bufdesc1.length = strlen (service) + strlen ("@")
+	  + strlen (hostname) + 1;
+	bufdesc1.value = malloc (bufdesc1.length);
+	if (bufdesc1.value == NULL)
+	  return GSASL_MALLOC_ERROR;
+
+	sprintf (bufdesc1.value, "%s@%s", service, hostname);
+
+	maj_stat = gss_import_name (&min_stat, &bufdesc1,
+				    GSS_C_NT_HOSTBASED_SERVICE, &server);
+	free (bufdesc1.value);
+	if (GSS_ERROR (maj_stat))
+	  return GSASL_GSSAPI_IMPORT_NAME_ERROR;
+
+	maj_stat = gss_acquire_cred (&min_stat, server, 0,
+				     GSS_C_NULL_OID_SET, GSS_C_ACCEPT,
+				     &state->cred, NULL, NULL);
+	gss_release_name (&min_stat, &server);
+	if (GSS_ERROR (maj_stat))
+	  return GSASL_GSSAPI_ACQUIRE_CRED_ERROR;
+      }
       state->step++;
       /* fall through */
 
@@ -162,21 +155,28 @@ _gsasl_gssapi_server_step (Gsasl_session * sctx,
       if (maj_stat != GSS_S_COMPLETE && maj_stat != GSS_S_CONTINUE_NEEDED)
 	return GSASL_GSSAPI_ACCEPT_SEC_CONTEXT_ERROR;
 
-      *output = malloc (bufdesc2.length);
-      if (!*output)
-	return GSASL_MALLOC_ERROR;
-      memcpy (*output, bufdesc2.value, bufdesc2.length);
-      *output_len = bufdesc2.length;
+      if (maj_stat == GSS_S_COMPLETE)
+	state->step++;
+
+      if (maj_stat == GSS_S_CONTINUE_NEEDED || bufdesc2.length > 0)
+	{
+	  *output = malloc (bufdesc2.length);
+	  if (!*output)
+	    return GSASL_MALLOC_ERROR;
+	  memcpy (*output, bufdesc2.value, bufdesc2.length);
+	  *output_len = bufdesc2.length;
+	}
 
       maj_stat = gss_release_buffer (&min_stat, &bufdesc2);
       if (GSS_ERROR (maj_stat))
 	return GSASL_GSSAPI_RELEASE_BUFFER_ERROR;
 
-      if (maj_stat == GSS_S_COMPLETE)
-	state->step++;
-
-      res = GSASL_NEEDS_MORE;
-      break;
+      if (maj_stat == GSS_S_CONTINUE_NEEDED || *output_len > 0)
+	{
+	  res = GSASL_NEEDS_MORE;
+	  break;
+	}
+      /* fall through */
 
     case 2:
       memset (tmp, 0xFF, 4);
@@ -244,6 +244,10 @@ _gsasl_gssapi_server_step (Gsasl_session * sctx,
       gsasl_property_set_raw (sctx, GSASL_GSSAPI_DISPLAY_NAME,
 			      client_name.value, client_name.length);
 
+      maj_stat = gss_release_buffer (&min_stat, &client_name);
+      if (GSS_ERROR (maj_stat))
+	return GSASL_GSSAPI_RELEASE_BUFFER_ERROR;
+
       maj_stat = gss_release_buffer (&min_stat, &bufdesc2);
       if (GSS_ERROR (maj_stat))
 	return GSASL_GSSAPI_RELEASE_BUFFER_ERROR;
