@@ -1,5 +1,5 @@
 /* obsolete.c --- Obsolete functions kept around for backwards compatibility.
- * Copyright (C) 2002-2012 Simon Josefsson
+ * Copyright (C) 2002-2020 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -23,7 +23,7 @@
 #include "internal.h"
 
 #if USE_DIGEST_MD5
-# include "qop.h"
+#include "qop.h"
 #endif
 
 /**
@@ -43,7 +43,7 @@
  * Deprecated: Use gsasl_client_mechlist() instead.
  **/
 int
-gsasl_client_listmech (Gsasl * ctx, char *out, size_t * outlen)
+gsasl_client_listmech (Gsasl * ctx, char *out, size_t *outlen)
 {
   char *tmp;
   int rc;
@@ -86,7 +86,7 @@ gsasl_client_listmech (Gsasl * ctx, char *out, size_t * outlen)
  * Deprecated: Use gsasl_server_mechlist() instead.
  **/
 int
-gsasl_server_listmech (Gsasl * ctx, char *out, size_t * outlen)
+gsasl_server_listmech (Gsasl * ctx, char *out, size_t *outlen)
 {
   char *tmp;
   int rc;
@@ -115,7 +115,7 @@ gsasl_server_listmech (Gsasl * ctx, char *out, size_t * outlen)
 static int
 _gsasl_step (Gsasl_session * sctx,
 	     const char *input, size_t input_len,
-	     char *output, size_t * output_len)
+	     char *output, size_t *output_len)
 {
   char *tmp;
   size_t tmplen;
@@ -166,7 +166,7 @@ _gsasl_step (Gsasl_session * sctx,
 int
 gsasl_client_step (Gsasl_session * sctx,
 		   const char *input,
-		   size_t input_len, char *output, size_t * output_len)
+		   size_t input_len, char *output, size_t *output_len)
 {
   return _gsasl_step (sctx, input, input_len, output, output_len);
 }
@@ -196,7 +196,7 @@ gsasl_client_step (Gsasl_session * sctx,
 int
 gsasl_server_step (Gsasl_session * sctx,
 		   const char *input,
-		   size_t input_len, char *output, size_t * output_len)
+		   size_t input_len, char *output, size_t *output_len)
 {
   return _gsasl_step (sctx, input, input_len, output, output_len);
 }
@@ -473,7 +473,7 @@ gsasl_ctx_get (Gsasl_session * sctx)
 int
 gsasl_encode_inline (Gsasl_session * sctx,
 		     const char *input, size_t input_len,
-		     char *output, size_t * output_len)
+		     char *output, size_t *output_len)
 {
   char *tmp;
   size_t tmplen;
@@ -513,7 +513,7 @@ gsasl_encode_inline (Gsasl_session * sctx,
 int
 gsasl_decode_inline (Gsasl_session * sctx,
 		     const char *input, size_t input_len,
-		     char *output, size_t * output_len)
+		     char *output, size_t *output_len)
 {
   char *tmp;
   size_t tmplen;
@@ -624,7 +624,8 @@ gsasl_appinfo_get (Gsasl_session * sctx)
  * that chose which mechanism to use.
  **/
 const char *
-gsasl_server_suggest_mechanism (Gsasl * ctx, const char *mechlist)
+gsasl_server_suggest_mechanism (Gsasl * ctx _GL_UNUSED,
+				const char *mechlist _GL_UNUSED)
 {
   return NULL;			/* This function is just silly. */
 }
@@ -1630,7 +1631,7 @@ gsasl_server_callback_service_get (Gsasl * ctx)
 }
 
 #if HAVE_LIBIDN
-# include <stringprep.h>
+#include <stringprep.h>
 #endif
 
 /**
@@ -1767,7 +1768,7 @@ gsasl_stringprep_trace (const char *in, int *stringprep_rc)
  **/
 int
 gsasl_md5pwd_get_password (const char *filename,
-			   const char *username, char *key, size_t * keylen)
+			   const char *username, char *key, size_t *keylen)
 {
   char *tmp;
   size_t tmplen;
@@ -2055,8 +2056,8 @@ _gsasl_obsolete_property_map (Gsasl_session * sctx, Gsasl_property prop)
 }
 
 int
-_gsasl_obsolete_callback (Gsasl * ctx, Gsasl_session * sctx,
-			  Gsasl_property prop)
+_gsasl_obsolete_callback (Gsasl * ctx _GL_UNUSED,
+			  Gsasl_session * sctx, Gsasl_property prop)
 {
   int res;
 
@@ -2092,7 +2093,7 @@ _gsasl_obsolete_callback (Gsasl * ctx, Gsasl_session * sctx,
       {
 	Gsasl_server_callback_securid cb_securid
 	  = gsasl_server_callback_securid_get (sctx->ctx);
-#define MAX_SECURID 32 /* See RFC 2808. */
+#define MAX_SECURID 32		/* See RFC 2808. */
 	char buf[MAX_SECURID + 1];
 	size_t buflen = MAX_SECURID;
 	if (!cb_securid)
@@ -2161,7 +2162,8 @@ _gsasl_obsolete_callback (Gsasl * ctx, Gsasl_session * sctx,
 	  break;
 	qops = cb_qop (sctx);
 	if (qops & 0x07)
-	  gsasl_property_set (sctx, GSASL_QOPS, digest_md5_qops2qopstr(qops));
+	  gsasl_property_set (sctx, GSASL_QOPS,
+			      digest_md5_qops2qopstr (qops));
 	return GSASL_OK;
 	break;
       }
@@ -2173,3 +2175,103 @@ _gsasl_obsolete_callback (Gsasl * ctx, Gsasl_session * sctx,
 
   return GSASL_NO_CALLBACK;
 }
+
+#include "gc.h"
+
+/**
+ * gsasl_md5:
+ * @in: input character array of data to hash.
+ * @inlen: length of input character array of data to hash.
+ * @out: newly allocated 16-byte character array with hash of data.
+ *
+ * Compute hash of data using MD5.  The @out buffer must be
+ * deallocated by the caller.
+ *
+ * Return value: Returns %GSASL_OK iff successful.
+ *
+ * Deprecated: Use a crypto library.
+ **/
+int
+gsasl_md5 (const char *in, size_t inlen, char *out[])
+{
+  *out = malloc (GC_MD5_DIGEST_SIZE);
+  if (!*out)
+    return GSASL_MALLOC_ERROR;
+  return gc_md5 (in, inlen, *out);
+}
+
+/**
+ * gsasl_hmac_md5:
+ * @key: input character array with key to use.
+ * @keylen: length of input character array with key to use.
+ * @in: input character array of data to hash.
+ * @inlen: length of input character array of data to hash.
+ * @outhash: newly allocated 16-byte character array with keyed hash of data.
+ *
+ * Compute keyed checksum of data using HMAC-MD5.  The @outhash buffer
+ * must be deallocated by the caller.
+ *
+ * Return value: Returns %GSASL_OK iff successful.
+ *
+ * Deprecated: Use a crypto library.
+ **/
+int
+gsasl_hmac_md5 (const char *key, size_t keylen,
+		const char *in, size_t inlen, char *outhash[])
+{
+  *outhash = malloc (GC_MD5_DIGEST_SIZE);
+  if (!*outhash)
+    return GSASL_MALLOC_ERROR;
+  return gc_hmac_md5 (key, keylen, in, inlen, *outhash);
+}
+
+/**
+ * gsasl_sha1:
+ * @in: input character array of data to hash.
+ * @inlen: length of input character array of data to hash.
+ * @out: newly allocated 20-byte character array with hash of data.
+ *
+ * Compute hash of data using SHA1.  The @out buffer must be
+ * deallocated by the caller.
+ *
+ * Return value: Returns %GSASL_OK iff successful.
+ *
+ * Since: 1.3
+ *
+ * Deprecated: Use a crypto library.
+ **/
+int
+gsasl_sha1 (const char *in, size_t inlen, char *out[])
+{
+  *out = malloc (GC_SHA1_DIGEST_SIZE);
+  if (!*out)
+    return GSASL_MALLOC_ERROR;
+  return gc_sha1 (in, inlen, *out);
+}
+
+/**
+ * gsasl_hmac_sha1:
+ * @key: input character array with key to use.
+ * @keylen: length of input character array with key to use.
+ * @in: input character array of data to hash.
+ * @inlen: length of input character array of data to hash.
+ * @outhash: newly allocated 20-byte character array with keyed hash of data.
+ *
+ * Compute keyed checksum of data using HMAC-SHA1.  The @outhash buffer
+ * must be deallocated by the caller.
+ *
+ * Return value: Returns %GSASL_OK iff successful.
+ *
+ * Since: 1.3
+ *
+ * Deprecated: Use a crypto library.
+ **/
+int
+gsasl_hmac_sha1 (const char *key, size_t keylen,
+		 const char *in, size_t inlen, char *outhash[])
+{
+  *outhash = malloc (GC_SHA1_DIGEST_SIZE);
+  if (!*outhash)
+    return GSASL_MALLOC_ERROR;
+  return gc_hmac_sha1 (key, keylen, in, inlen, *outhash);
+}
